﻿using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace DocIO_Test
{
    class Program
    {
        static void Main(string[] args)
        {
            MainAsync(args).GetAwaiter().GetResult();
        }

        public static async Task MainAsync(string[] args)
        {
            await Task.Delay(1);
            var filename = "TestDocument.docx";
            GenerateDocument(filename);
        }


        public static void GenerateDocument(string filename)
        {
            var exePath = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
            var fullpath = exePath + $"\\{filename}";
            var file = File.ReadAllBytes(fullpath);
            var docParameters = GetDocParameters();
            using (Stream stream = new MemoryStream(file))
            {
                using (var document = new WordDocument(stream, FormatType.Automatic))
                {
                    document.MailMerge.RemoveEmptyGroup = true;
                    document.MailMerge.ExecuteNestedGroup(docParameters.DataSet, docParameters.Commands);

                    var generatedFileName = GetGeneratedFileName(filename);
                    var filepath = GetFormFile(generatedFileName);

                    using (var filestream = new FileStream(filepath, FileMode.Create))
                    {
                        document.Save(filestream, FormatType.Docx);
                    }
                }
            }
        }


        public static DocumentParameterDTO GetDocParameters()
        {
            var docParameters = new DocumentParameterDTO("Records", Guid.NewGuid().ToString());
            docParameters.Commands.Add(new DictionaryEntry("Records", string.Empty));
            docParameters.Commands.Add(new DictionaryEntry("Bicycles", "RecordID = %Records.RecordID%"));
            docParameters.Commands.Add(new DictionaryEntry("Customparts", "BicycleID = %Bicycles.BicycleID%"));

            var valueObjects = new Dictionary<string, List<Dictionary<string, object>>>();

            //Policies
            var valueObject = new List<Dictionary<string, object>>();
            var localObject = new Dictionary<string, object>();
            var localObject2 = new Dictionary<string, object>();
            var valueObject2 = new List<Dictionary<string, object>>();

            localObject.Add("RecordID", 1);
            localObject.Add("PersonFirstName", "Ootam");
            localObject.Add("PersonSurname", "Seewoogoolam");
            localObject.Add("PersonDateOfBirth", "1980-05-20");
            localObject.Add("PersonPostCode", "75301");
            localObject.Add("PersonStreetAddress", "James Street");
            localObject.Add("PersonTown", "Phoenix");
            localObject.Add("PersonCountry", "South Africa");
            valueObject.Add(localObject);
            valueObjects.Add("Records", valueObject);

            //Bicycles
            valueObject = new List<Dictionary<string, object>>();
            localObject = new Dictionary<string, object>();
            localObject.Add("RecordID", 1);
            localObject.Add("BicycleID", 1);
            localObject.Add("BicycleType", "test1");
            localObject.Add("BicycleMake", "test1");
            localObject.Add("BicycleModel", "test1");
            localObject.Add("BicycleYear", "test1");
            localObject.Add("BicycleSerial", "test1");
            valueObject.Add(localObject);


            localObject = new Dictionary<string, object>();
            localObject.Add("RecordID", 1);
            localObject.Add("BicycleID", 2);
            localObject.Add("BicycleType", "test2");
            localObject.Add("BicycleMake", "test2");
            localObject.Add("BicycleModel", "test2");
            localObject.Add("BicycleYear", "test2");
            localObject.Add("BicycleSerial", "test2");
            valueObject.Add(localObject);
            valueObjects.Add("Bicycles", valueObject);

            //Customparts
            valueObject = new List<Dictionary<string, object>>();
            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 1);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test2");
            localObject.Add("Type", "test2");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 2);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test1");
            localObject.Add("Type", "test1");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 3);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test3");
            localObject.Add("Type", "test3");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 4);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test4");
            localObject.Add("Type", "test4");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 5);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test5");
            localObject.Add("Type", "test5");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 6);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test6");
            localObject.Add("Type", "test6");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 7);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test7");
            localObject.Add("Type", "test7");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 8);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test8");
            localObject.Add("Type", "test8");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 9);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test9");
            localObject.Add("Type", "test9");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 10);
            localObject.Add("BicycleID", 1);
            localObject.Add("Description", "test10");
            localObject.Add("Type", "test10");
            valueObject.Add(localObject);

            localObject = new Dictionary<string, object>();
            localObject.Add("CustompartID", 11);
            localObject.Add("BicycleID", 2);
            localObject.Add("Description", "test11");
            localObject.Add("Type", "test11");
            valueObject.Add(localObject);

            valueObjects.Add("Customparts", valueObject);

            foreach (var valObject in valueObjects)
            {
                docParameters.DataSet.Add(new MailMergeDataTable(valObject.Key, valObject.Value));
            }

            return docParameters;
        }

        public static string GetGeneratedFileName(string fileName)
        {
            var splitFileName = fileName.Split('.');
            var generatedfilename = string.Empty;
            for (int i = 0; i < splitFileName.Length; i++)
            {
                if (i != splitFileName.Length - 1)
                {
                    generatedfilename += $"{splitFileName[i]}.";
                }
            }
            generatedfilename += $"{UnixDateTimeExtension.GetUnixTimeNow()}.docx";

            return generatedfilename;
        }

        public static string GetFormFile(string filename)
        {
            var exePath = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
            var fullpath = exePath + "\\" + filename;
            return fullpath;
        }
    }
}
